'' =================================================================================================
''
''   File....... jm_wav_player_demo.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013-2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 12 FEB 2015
''
'' =================================================================================================


con  { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con  { io pins }

  RX1    = 31  { i }                                             ' programming / terminal
  TX1    = 30  { o }                                              
                                                                  
  SDA    = 29  { io }                                            ' eeprom / i2c
  SCL    = 28  { io }                                             
                                                                  
  A_RT   = 27  { o }                                             ' audio outputs
  A_LF   = 26  { o }                                              
                                                                  
  SD_CS  = 25  { o }                                             '  PAB microSD connections   
  SD_DI  = 24  { o }                                              
  SD_CLK = 23  { o }
  SD_DO  = 22  { i }
  

con 

  #(-1), IS_ON, IS_OFF
  

con  { PST formatting }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR 
  #14, GOTOX, GOTOY, CLS


obj

  audio : "jm_wav_player_2015"  
  term  : "jm_fullduplexserial"


var


dat

  {
    Parameters for audio setup

    L/R channel outputs        (-1 if channel not used [for mono])
    SD card connections        (4 pins)
    Dither for stereo_dacs cog (1 [most] to 8 [least])
  }

  AudioSetup            long    A_LF, A_RT
                        long    SD_DO, SD_CLK, SD_DI, SD_CS
                        long    3                                ' tune for system, 1..8

  
  AudioFile             byte    "shot.wav", 0                    ' must conform to DOS 8.3 name

  
  Menu                  byte    "-------------------", CR
                        byte    " Audio Player Demo ", CR
                        byte    "-------------------", CR
                        byte    CR
                        byte    "[1] Single shot    ", CR                          
                        byte    "[2] Machine gun    ", CR
                        byte    0 

                        
  Report                byte    "--------------------------", CR
                        byte    " Atrributes:              ", CR
                        byte    "--------------------------", CR
                        byte    CR
                        byte    "Channels......            ", CR
                        byte    "Sample Rate...            ", CR
                        byte    "Duration......            ", CR
                        byte    CR
                        byte    "Press a key."
                        byte    0
  

pub main | c

  audio.start(@AudioSetup)                                       ' start wav player
  audio.set_volume(5, 5)                                       ' set volume for system

  term.start(RX1, TX1, %0000, 115_200)                           ' start serial io for terminal
  pause(10)

  ' open pst and press a key to start

  term.rxflush
  term.rx

  show_report(@AudioFile)                                        ' show audio file characteristics

  repeat
    term.rxflush                                                 ' clear keyboard buffer
    term.tx(CLS)                                                 ' clear screen
    term.str(@Menu)                                              ' display menu
                                                                  
    c := ucase(term.rx)                                          ' wait for user input
                                                                  
    case c                                                        
      "1":                                                        
        audio.play(@AudioFile, 1)                                ' show audio file attributes
                                                                  
      "2":
        repeat 20                                                      
          audio.play(@AudioFile, 1)
          pause(60)                                              ' 1000 rounds per minute                            



pub show_report(p_filename) | check, secs

'' Opens file (if possible) and shows basic parameters

  term.tx(CLS)

  check := audio.get_header(p_filename)                          ' load header from file
  if (check < 0)                                                 ' if any error
    term.str(string("Cannot open "))                             '  alert user
    term.str(p_filename)                                          
    term.rxflush                                                  
    term.rx                                                       
                                                                  
  term.str(@Report)                                               
                                                                  
  move_to(13, 1)                                                  
  term.str(p_filename)                                           ' display file name
                                                                  
  move_to(15, 4)                                                  
  term.dec(audio.channels)                                       ' display # of channels
                                                                  
  move_to(15, 5)                                                  
  term.dec(audio.sample_rate)                                    ' display audio sample rate
                                                                  
  move_to(15, 6)                                                  
  secs := audio.duration                                         ' duration in 0.01s units
  term.dec(secs / 100)                                            
  term.tx(".")
  term.rjdec(secs // 100, 2, "0")

  term.rxflush
  term.rx


pub ucase(c)

'' If c is lowercase, convert to uppercase

  if ((c => "a") and (c =< "z"))
    c -= 32

  return c
  

pub move_to(x, y)

'' Move cursor for PST

  term.tx(GOTOXY)
  term.tx(x)
  term.tx(y)
  

con

  { --------------- }
  {   B A S I C S   }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat  { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}